/* -----------------------------------------------------------------------------
	viewText -- a GTK+ based, lightweight, and fast textviewer
	Copyright (C) 2008 Andreas Rönnquist

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
--------------------------------------------------------------------------------
	Menu.cpp

----------------------------------------------------------------------------- */
#include <gtk/gtk.h>

#include "Menu.h"

#include "File.h"
#include "Edit.h"
#include "Dialog.h"
#include "Options.h"

/* -----------------------------------------------------------------------------
	InitMenu
----------------------------------------------------------------------------- */
void Menu::InitMenu(GtkWidget *window)
{
	GtkWidget *menubar,*file,/**edit,*/*options,*help,*filemenu,*editmenu,*helpmenu,*optionsmenu;

	GtkWidget *newitem,*open,*save,*save_as,*quit;
	GtkWidget *cutter,*copy,*paste,*about; //,*contents
	//GtkWidget *showStatus;
	GtkWidget *separator;
	GtkAccelGroup *group;

	GtkWidget *changeFont,*changeColors;

	group=gtk_accel_group_new();

	menubar=gtk_menu_bar_new();

	gtk_box_pack_start (GTK_BOX (window), menubar, FALSE, TRUE, 0);

	gtk_menu_bar_set_pack_direction(GTK_MENU_BAR(menubar),GTK_PACK_DIRECTION_LTR);

	file=gtk_menu_item_new_with_label("File");
	//edit=gtk_menu_item_new_with_label("Edit");
	options=gtk_menu_item_new_with_label("Options");
	help=gtk_menu_item_new_with_label("Help");

	filemenu=gtk_menu_new();
	editmenu=gtk_menu_new();
	optionsmenu=gtk_menu_new();
	helpmenu=gtk_menu_new();

	gtk_menu_item_set_submenu(GTK_MENU_ITEM(file),filemenu);
	//gtk_menu_item_set_submenu(GTK_MENU_ITEM(edit),editmenu);
	gtk_menu_item_set_submenu(GTK_MENU_ITEM(options),optionsmenu);
	gtk_menu_item_set_submenu(GTK_MENU_ITEM(help),helpmenu);

	gtk_menu_shell_append(GTK_MENU_SHELL(menubar),file);
	//gtk_menu_shell_append(GTK_MENU_SHELL(menubar),edit);
	gtk_menu_shell_append(GTK_MENU_SHELL(menubar),options);
	gtk_menu_shell_append(GTK_MENU_SHELL(menubar),help);

	// File menu
	newitem=gtk_image_menu_item_new_from_stock(GTK_STOCK_NEW,group);
	open=gtk_image_menu_item_new_from_stock(GTK_STOCK_OPEN,group);
	save=gtk_image_menu_item_new_from_stock(GTK_STOCK_SAVE,group);
	save_as=gtk_image_menu_item_new_from_stock(GTK_STOCK_SAVE_AS,group);
	separator=gtk_separator_menu_item_new();
	quit=gtk_image_menu_item_new_from_stock(GTK_STOCK_QUIT,group);

	//gtk_menu_shell_append(GTK_MENU_SHELL(filemenu),newitem);
	gtk_menu_shell_append(GTK_MENU_SHELL(filemenu),open);
	//gtk_menu_shell_append(GTK_MENU_SHELL(filemenu),save);
	//gtk_menu_shell_append(GTK_MENU_SHELL(filemenu),save_as);
	gtk_menu_shell_append(GTK_MENU_SHELL(filemenu),separator);
	gtk_menu_shell_append(GTK_MENU_SHELL(filemenu),quit);

	g_signal_connect_swapped(G_OBJECT(newitem),"activate",G_CALLBACK(File::NewOptionClicked),(gpointer)group);
	g_signal_connect_swapped(G_OBJECT(open),"activate",G_CALLBACK(File::OpenOptionClicked),(gpointer)group);
	g_signal_connect_swapped(G_OBJECT(save),"activate",G_CALLBACK(File::SaveOptionClicked),(gpointer)group);
	g_signal_connect_swapped(G_OBJECT(save_as),"activate",G_CALLBACK(File::SaveAsOptionClicked),(gpointer)group);
	g_signal_connect_swapped(G_OBJECT(quit),"activate",G_CALLBACK(File::QuitOptionClicked),(gpointer)group);

	// Options menu

	changeFont=gtk_image_menu_item_new_from_stock(GTK_STOCK_SELECT_FONT,group);
	changeColors=gtk_image_menu_item_new_from_stock(GTK_STOCK_SELECT_COLOR,group);

	gtk_menu_shell_append(GTK_MENU_SHELL(optionsmenu),changeFont);
	gtk_menu_shell_append(GTK_MENU_SHELL(optionsmenu),changeColors);

	g_signal_connect_swapped(G_OBJECT(changeFont),"activate",G_CALLBACK(Options::ChangeFont),(gpointer)group);
	g_signal_connect_swapped(G_OBJECT(changeColors),"activate",G_CALLBACK(Options::ChangeColors),(gpointer)group);

	//showStatus=gtk_image_menu

	// Edit menu
	cutter=gtk_image_menu_item_new_from_stock(GTK_STOCK_CUT,group);
	copy=gtk_image_menu_item_new_from_stock(GTK_STOCK_COPY,group);
	paste=gtk_image_menu_item_new_from_stock(GTK_STOCK_PASTE,group);

	gtk_menu_shell_append(GTK_MENU_SHELL(editmenu),cutter);
	gtk_menu_shell_append(GTK_MENU_SHELL(editmenu),copy);
	gtk_menu_shell_append(GTK_MENU_SHELL(editmenu),paste);

	g_signal_connect_swapped(G_OBJECT(cutter),"activate",G_CALLBACK(Edit::CutOptionClicked),(gpointer)group);
	g_signal_connect_swapped(G_OBJECT(copy),"activate",G_CALLBACK(Edit::CopyOptionClicked),(gpointer)group);
	g_signal_connect_swapped(G_OBJECT(paste),"activate",G_CALLBACK(Edit::PasteOptionClicked),(gpointer)group);

	// Help menu
	about=gtk_image_menu_item_new_from_stock(GTK_STOCK_ABOUT,group);
	gtk_menu_shell_append(GTK_MENU_SHELL(helpmenu),about);
	g_signal_connect_swapped(G_OBJECT(about),"activate",G_CALLBACK(Dialog::AboutDialog),(gpointer)group);
}
