/* -----------------------------------------------------------------------------
	viewText -- a GTK+ based, lightweight, and fast textviewer
	Copyright (C) 2008 Andreas Rönnquist

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
--------------------------------------------------------------------------------
	TextView.cpp

		Code for the actual TextView widget

----------------------------------------------------------------------------- */
#include <string>
#include <fstream>
#include <iostream>
#include <sstream>

#include <gtk/gtk.h>

#include "Program.h"
#include "Statusbar.h"
#include "TextView.h"
#include "Dialog.h"

#include "Main.h"

#define BROWSER_CMDLINE "firefox"

/* -----------------------------------------------------------------------------

----------------------------------------------------------------------------- */
gboolean TextView::TextViewMotionNotify(GtkWidget *textview, GdkEventMotion *event, /*__attribute__((unused))*/gpointer d)
{
	if (event->type==(GdkEventType)GDK_MOTION_NOTIFY) {

		gint x=0, y=0;
		GtkTextIter iter;

		gtk_text_view_window_to_buffer_coords(GTK_TEXT_VIEW(textview), GTK_TEXT_WINDOW_WIDGET,
							(gint)event->x, (gint)event->y, &x, &y);

		gtk_text_view_get_iter_at_location(GTK_TEXT_VIEW(textview), &iter, x, y);


		GSList *list=gtk_text_iter_get_tags(&iter);

		if (g_slist_length(list)!=0) {
			gdk_window_set_cursor(event->window, Main::handCursor);

			GtkTextTag *item=(GtkTextTag*)(list->data);

			Statusbar::SetText(item->name);

		} else {
			gdk_window_set_cursor(event->window, Main::standardCursor);

			Statusbar::SetText(Program::StandardText);
		}

		gdk_window_get_pointer(event->window, NULL, NULL, NULL);

		return true;
	}

	return false;
}

/* -----------------------------------------------------------------------------

----------------------------------------------------------------------------- */
TextView::TextView(std::string filename)
{
	GtkTextTagTable *tagTbl;

	textview=gtk_text_view_new();
	textBuffer=gtk_text_view_get_buffer(GTK_TEXT_VIEW(textview));
	//gtk_text_buffer_set_text(buffer,"GoonEd, the texteditor.... \n krroth",-1);

	gtk_text_view_set_wrap_mode(GTK_TEXT_VIEW(textview),GTK_WRAP_NONE);

	gtk_text_view_set_editable(GTK_TEXT_VIEW(textview),TRUE);

	gtk_text_view_set_cursor_visible(GTK_TEXT_VIEW(textview),TRUE);


	tagTbl=gtk_text_buffer_get_tag_table(textBuffer);

	int result=-1;

	if (filename!="") {
		result=LoadFile(filename);
	}

#ifdef G_OS_WIN32
	SetFontString("Courier 8");
#else
	SetFontString("Monospace 8");
#endif

	if (result==0) FixLinks();

}

void TextView::FixLinks()
{
	//tag =
	gboolean foundsomething=FALSE;

	GtkTextIter begin,end;
	GtkTextIter searchIter;

	GtkTextIter textEnd;

	gtk_text_buffer_get_end_iter(textBuffer,&textEnd);

	gtk_text_buffer_get_iter_at_line (textBuffer, &searchIter, 0);
	gtk_text_buffer_get_iter_at_line (textBuffer, &end, 0);

	int co=0;

	std::stringstream tempText;

	gchar *buf;

	GtkTextTagTable *tagTable=gtk_text_buffer_get_tag_table(textBuffer);

	do {

		foundsomething=gtk_text_iter_forward_search(&searchIter,"http",GTK_TEXT_SEARCH_VISIBLE_ONLY,&begin/*&start*/,&end/*&end*/,&textEnd);

		if (foundsomething) {

			GtkTextIter test=begin;
			gtk_text_iter_forward_find_char(&test,&TextView::FindLinkSeparator,NULL,NULL);

			/*
			tempText.str("");

			tempText << buf << co << 'k';

			gchar *link_text=(gchar*)(tempText.str().c_str());

			tempText.str("");
			*/

	//		gchar *link_text=buf;

			buf=gtk_text_buffer_get_text(textBuffer,&begin,&test,TRUE);

			GtkTextTag *tag=gtk_text_tag_table_lookup(tagTable,buf);

			if (tag==NULL) {

				//std::cout << "Tag added:" << buf << std::endl;

				tag=gtk_text_buffer_create_tag(textBuffer, buf,
										"foreground", "darkgreen",
										"weight", PANGO_WEIGHT_NORMAL,
										"underline", PANGO_UNDERLINE_SINGLE,NULL);

					g_signal_connect(G_OBJECT(tag), "event",
				 G_CALLBACK(ActivateUrl), NULL);

			}

			gtk_text_buffer_apply_tag (textBuffer, GTK_TEXT_TAG(tag), &begin, &test);

			searchIter=test;

			co++;

			tagTable=gtk_text_buffer_get_tag_table(textBuffer);
		}

	} while(foundsomething);

}


/* -----------------------------------------------------------------------------
	LoadFile

			based on code from Xfce's mousepad
----------------------------------------------------------------------------- */
int TextView::LoadFile(std::string filename)
{
	gchar *contents;
	gsize length;
	GError *err = NULL;
	const gchar *charset;
	gchar *str = NULL;
	GtkTextIter iter;

	//Dialog::RunDialogMessage(gtk_widget_get_toplevel(Main::window), GTK_MESSAGE_ERROR, err->message);

	gchar *fname=(gchar*)(filename.c_str());

	textBuffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(textview));


	if (!g_file_get_contents(fname, &contents, &length, &err)) {

		//std::cout << "Error!" << std::endl;
		//if (g_file_test(fname, G_FILE_TEST_EXISTS)) {
			//std::cout << "Error:" << err->message << std::endl;


			Dialog::RunDialogMessage(gtk_widget_get_toplevel(Main::window), GTK_MESSAGE_ERROR, err->message);
			g_error_free(err);
			return -1;
	/*
		g_error_free(err);
		err = NULL;
		contents = g_strdup("");
		*/

		//}

	}

	charset = "ISO-8859-1";
	/*

	fi->lineend = detect_line_ending(contents);
	if (fi->lineend != LF)
		convert_line_ending_to_lf(contents);

	if (fi->charset)
		charset = fi->charset;
	else
		charset = detect_charset(contents);
	if (!charset) {
		if (fi->manual_charset)
			charset = fi->manual_charset;
		else
			charset = get_default_charset();
	}
	*/

	if (length)
		do {
			if (err) {
				if (strcmp(charset, "GB2312") == 0)
					charset = "GB18030";
				else if (strcasecmp(charset, "BIG5") == 0)
					charset = "CP950";
				else if (strcasecmp(charset, "CP950") == 0)
					charset = "BIG5-HKSCS";
				else if (strcasecmp(charset, "EUC-KR") == 0)
					charset = "CP949";
				else if (strcasecmp(charset, "CP949") == 0)
					charset = "CP1361";
				else if (strcasecmp(charset, "VISCII") == 0)
					charset = "CP1258";
				else if (strcasecmp(charset, "TIS-620") == 0)
					charset = "CP874";
				else
					charset = "ISO-8859-1";
				g_error_free(err);
				err = NULL;
			}
			str = g_convert(contents, -1, "UTF-8", charset, NULL, NULL, &err);
		} while (err);
	else
		str = g_strdup("");

/*
	if (charset != fi->charset) {
		g_free(fi->charset);
		fi->charset = g_strdup(charset);
	}
	if (fi->manual_charset)
		if (strcmp(fi->manual_charset, fi->charset) != 0) {
			g_free(fi->manual_charset);
			fi->manual_charset = NULL;
		}
		*/
	g_free(contents);

		//undo_disconnect_signal(textbuffer);

//	undo_block_signal(textbuffer);
	gtk_text_buffer_set_text(textBuffer, "", 0);
	gtk_text_buffer_get_start_iter(textBuffer, &iter);
	gtk_text_buffer_insert(textBuffer, &iter, str, strlen(str));
	gtk_text_buffer_get_start_iter(textBuffer, &iter);
	gtk_text_buffer_place_cursor(textBuffer, &iter);
	gtk_text_buffer_set_modified(textBuffer, FALSE);
	//gtk_text_view_scroll_to_iter(GTK_TEXT_VIEW(textview), &iter, 0, FALSE, 0, 0);
	g_free(str);
//	undo_unblock_signal(textbuffer);
	return 0;
}

/* -----------------------------------------------------------------------------
	Destructor
		make sure you free used memory
----------------------------------------------------------------------------- */
TextView::~TextView()
{
	//if (text!=0) free(text);
}

/* -----------------------------------------------------------------------------
	ContainerAdd
		Adds the textview to an GTK container
----------------------------------------------------------------------------- */
void TextView::ContainerAdd(GtkContainer *container)
{
	gtk_container_add(GTK_CONTAINER(container),textview);
}

/* -----------------------------------------------------------------------------

----------------------------------------------------------------------------- */
GtkTextBuffer *TextView::GetTextBuffer()
{
	return textBuffer;
}

/* -----------------------------------------------------------------------------

----------------------------------------------------------------------------- */
GtkTextView *TextView::GetTextView()
{
	return GTK_TEXT_VIEW(textview);
}

/* -----------------------------------------------------------------------------

----------------------------------------------------------------------------- */
gboolean TextView::FindLinkSeparator(gunichar ch,gpointer user_data)
{
	switch (ch)
	{
	case ' ':
	case ',':
	case ';':
	case '\n':
	case '"':
	case '\r':
	case '<':
	case '>':
		return TRUE;
	default:
		return FALSE;
	}
}

/* -----------------------------------------------------------------------------

----------------------------------------------------------------------------- */
void TextView::SetFontString(std::string fontName)
{
	PangoFontDescription *font_desc;
	font_desc = pango_font_description_from_string ((char*)(fontName.c_str()));
	gtk_widget_modify_font (textview, font_desc);
	pango_font_description_free (font_desc);

	m_FontString=fontName;
}

/* -----------------------------------------------------------------------------

----------------------------------------------------------------------------- */
std::string TextView::GetFontString()
{
	return m_FontString;
}

/* -----------------------------------------------------------------------------

----------------------------------------------------------------------------- */
gboolean TextView::ActivateUrl(GtkTextTag *tag, __attribute__((unused))GObject *arg1, GdkEvent *event, GtkTextIter *arg2, __attribute__((unused))gpointer d){
	GdkEventButton *event_btn=(GdkEventButton*)event;
	if (event->type == GDK_BUTTON_RELEASE && event_btn->button == 1){
		GtkTextIter start, end;
		gchar *link;

		if (gtk_text_iter_toggles_tag(arg2, tag)) return false;

		/* Go backward until this tag begins */
		start=*arg2;
		while (gtk_text_iter_begins_tag(&start, tag) == false) gtk_text_iter_backward_char(&start);

		/* Go backward until this tag ends */
		end=*arg2;
		while (gtk_text_iter_ends_tag(&end, tag) == false) gtk_text_iter_forward_char(&end);

		link=gtk_text_iter_get_text(&start, &end);

		if (link){
			gint memNeeded=strlen(BROWSER_CMDLINE)+strlen(link)+2;
			gchar *cmd_line=(gchar*)malloc(memNeeded);
			snprintf(cmd_line, memNeeded,"%s %s",BROWSER_CMDLINE, link);
			system(cmd_line);
		}
	}
	return false;
}
